# Copyright 2024 Zoé Cassiopée Gauthier.
#
# Use of this source code is governed by an MIT-style
# license that can be found in the LICENSE file or at
# https://opensource.org/licenses/MIT.

import argparse
import logging
import operator
import sys
from functools import reduce

from rich.logging import RichHandler

from pogo_scaled_estimators._version import version
from pogo_scaled_estimators.calculator import Calculator, Filter


def main_cli():
    parser = argparse.ArgumentParser()
    parser.add_argument("type", nargs="+", help="an attacker type")
    parser.add_argument("--level", type=int, default=40)
    parser.add_argument("--party", type=int, default=1)
    parser.add_argument("--no-legacy", dest="filters", action="append_const", const=Filter.DISALLOW_LEGACY_MOVES)
    parser.add_argument("--no-mega", dest="filters", action="append_const", const=Filter.DISALLOW_MEGA_POKEMON)
    parser.add_argument("--no-shadow", dest="filters", action="append_const", const=Filter.DISALLOW_SHADOW_POKEMON)
    parser.add_argument(
        "--no-legendary", dest="filters", action="append_const", const=Filter.DISALLOW_LEGENDARY_POKEMON
    )
    parser.add_argument(
        "-v", "--verbose", action="store_const", dest="log_level", const=logging.DEBUG, default=logging.WARNING
    )
    parser.add_argument("--version", action="version", version=version)

    args = parser.parse_args()

    logging.basicConfig(level=args.log_level, handlers=[RichHandler()])
    log = logging.getLogger()
    requests_log = logging.getLogger("urllib3")
    requests_log.setLevel(args.log_level)
    requests_log.propagate = True

    filters = reduce(operator.or_, args.filters or [], Filter.NO_FILTER)
    try:
        calculator = Calculator(args.type, filters)
        calculator.calculate(level=args.level, party=args.party)
    except Exception:
        log.exception("Could not calculate ASE.")


if __name__ == "__main__":
    sys.exit(main_cli())
